@@ -82,6 +82,16 @@
                  (const :tag "Oldest first" nil))
   :group 'jira)
 
+(defcustom jira-detail-reuse-buffer
+  nil
+  "If non-nil, reuse a single Jira detail buffer instead of creating separate
+buffers for each issue.
+When enabled, all issue details will be displayed in one buffer named
+ \"*Jira Issue Detail*\" instead of creating separate buffers
+like \"*Jira Issue Detail: [PROJ-123]*\"."
+  :type 'boolean
+  :group 'jira)
+
 ;; they override the default formatters specified
 ;; in jira-issues-fields
 (defvar jira-detail-formatters
@@ -181,7 +191,7 @@
 
 (cl-defun jira-detail--issue (key issue)
   "Show the detailed information of the ISSUE with KEY."
-  (with-current-buffer (get-buffer-create (concat "*Jira Issue Detail: [" key "]*"))
+  (with-current-buffer (jira-detail--get-issue-buffer key)
     (jira-detail)
     (setq jira-detail--current-key key)
     (setq jira-detail--current issue)
@@ -227,6 +237,7 @@
             (insert "\n\n")))))
     (jira-detail--show-attachments key issue)
     (jira-detail--show-subtasks key issue)
+    (jira-detail--show-linked-issues key issue)
     (jira-detail--show-other key)
     (pop-to-buffer (current-buffer))))
 
@@ -259,7 +270,7 @@
 
 (defun jira-detail--comments (key comments)
   "Format and insert COMMENTS from issue KEY."
-  (with-current-buffer (get-buffer-create (concat "*Jira Issue Detail: [" key "]*"))
+  (with-current-buffer (jira-detail--get-issue-buffer key)
     (let ((inhibit-read-only t))
       (goto-char (point-max))
       (magit-insert-section (jira-issue-comments nil nil)
@@ -291,7 +302,7 @@
   (let* ((fields (alist-get 'fields issue))
          (subtasks (alist-get 'subtasks fields)))
     (when (and subtasks (sequencep subtasks) (> (length subtasks) 0))
-      (with-current-buffer (get-buffer-create (concat "*Jira Issue Detail: [" key "]*"))
+      (with-current-buffer (jira-detail--get-issue-buffer key)
         (let ((inhibit-read-only t))
           (goto-char (point-max))
           (magit-insert-section (jira-issue-subtasks nil nil)
@@ -299,17 +310,68 @@
               (magit-insert-heading "🔗 Subtasks")
               (magit-insert-section-body
 		(seq-doseq (subtask subtasks)
-                  (let* ((subtask-key (alist-get 'key subtask))
-			 (subtask-fields (alist-get 'fields subtask))
-			 (subtask-summary (alist-get 'summary subtask-fields))
-			 (subtask-status (alist-get 'status subtask-fields)))
-                    (when (and (stringp subtask-key) (stringp subtask-summary))
-                      (insert (format " - %s  %s %s\n"
-				      (jira-fmt-issue-key-not-tabulated subtask-key)
-				      (jira-fmt-issue-status subtask-status)
-                                      subtask-summary))))))
+                  (jira-detail--format-issue-entry subtask)))
               (insert "\n"))))))))
 
+(defun jira-detail--get-issue-buffer (key)
+  "Get or create the Jira issue detail buffer for KEY.
+If `jira-detail-reuse-buffer' is enabled, reuse a single buffer for all issues."
+  (if jira-detail-reuse-buffer
+      (get-buffer-create "*Jira Issue Detail*")
+    (get-buffer-create (concat "*Jira Issue Detail: [" key "]*"))))
+
+(defun jira-detail--process-link (link)
+  "Process a single LINK and return (issue direction link-type) tuple."
+  (let* ((link-type (alist-get 'type link))
+         (outward-issue (alist-get 'outwardIssue link))
+         (inward-issue (alist-get 'inwardIssue link)))
+    (cond
+     (outward-issue (list outward-issue "outward" link-type))
+     (inward-issue (list inward-issue "inward" link-type)))))
+
+(defun jira-detail--show-linked-issues (key issue)
+  "Display linked issues for ISSUE with key KEY."
+  (let* ((fields (alist-get 'fields issue))
+         (issuelinks (alist-get 'issuelinks fields)))
+    (when (and issuelinks (sequencep issuelinks) (> (length issuelinks) 0))
+      (with-current-buffer (jira-detail--get-issue-buffer key)
+        (let ((inhibit-read-only t))
+          (goto-char (point-max))
+          (magit-insert-section (jira-issue-linked-issues nil nil)
+            (magit-insert-section (linked-issues-list nil nil)
+              (magit-insert-heading "🔗 Linked Issues")
+              (magit-insert-section-body
+                (seq-doseq (link issuelinks)
+                  (let* ((processed (jira-detail--process-link link))
+                         (issue (car processed))
+                         (direction (cadr processed))
+                         (link-type (caddr processed)))
+                    (when issue
+                      (jira-detail--format-linked-issue issue link-type direction)))))
+              (insert "\n"))))))))
+
+(defun jira-detail--format-issue-entry (issue &optional link-text)
+  "Format a single ISSUE entry, optionally with LINK-TEXT for linked issues.
+This is a shared function used by both subtasks and linked issues."
+  (let* ((issue-key (alist-get 'key issue))
+         (issue-fields (alist-get 'fields issue))
+         (issue-summary (alist-get 'summary issue-fields))
+         (issue-status (alist-get 'status issue-fields)))
+    (if (and (stringp issue-key) (stringp issue-summary))
+        (insert (format " - %s %s %s %s\n"
+                        (if link-text (concat "[" link-text "]") "")
+                        (jira-fmt-issue-key-not-tabulated issue-key)
+                        (jira-fmt-issue-status issue-status)
+                        issue-summary))
+      (insert (format " - [Invalid issue data: %s]\n" issue-key)))))
+
+(defun jira-detail--format-linked-issue (issue link-type direction)
+  "Format a single linked ISSUE with LINK-TYPE and DIRECTION."
+  (let* ((link-text (if (string= direction "outward")
+                        (alist-get 'outward link-type)
+                      (alist-get 'inward link-type))))
+    (jira-detail--format-issue-entry issue (or link-text "[Unknown link type]"))))
+
 (defvar-keymap jira-attachment-section-map
   :doc "Keymap for Jira attachment sections."
   "<RET>" #'jira-detail--get-attachment)
@@ -322,14 +384,14 @@
   (let* ((fields (alist-get 'fields issue))
          (attachments (alist-get 'attachment fields)))
     (when (> (length attachments) 0)
-      (with-current-buffer (get-buffer-create (concat "*Jira Issue Detail: [" key "]*"))
+      (with-current-buffer (jira-detail--get-issue-buffer key)
 	(let ((inhibit-read-only t))
           (goto-char (point-max))
           (magit-insert-section (jira-issue-attachments nil nil)
 	    (magit-insert-section (attachements-list nil nil)
               (magit-insert-heading "📎 Attachments (press RET to visualize)")
               (magit-insert-section-body
-		(mapcar
+		(mapc
 		 (lambda (attachment)
 		   (let* ((url (url-generic-parse-url (alist-get 'content attachment)))
                           ;; FIXME: verify that filename matches
@@ -365,6 +427,7 @@
     (_ (error "Not a Jira attachment"))))
 
 (defun jira-detail--show-attachment (name data)
+  "Show attachment NAME with DATA in a new buffer."
   (let ((x (generate-new-buffer-name name)))
     (pop-to-buffer x)
     (insert data)
@@ -378,11 +441,12 @@
     (set-buffer-modified-p t)))
 
 (defun jira-detail--show-other (key)
+  "Show other issue details for KEY."
   (jira-detail--watchers
    key
    (lambda (data _response)
      (jira-detail--show-comments key)
-     (with-current-buffer (get-buffer-create (concat "*Jira Issue Detail: [" key "]*"))
+     (with-current-buffer (jira-detail--get-issue-buffer key)
        (let ((inhibit-read-only t)
              (names (mapcar (lambda (u)
                               (alist-get 'displayName u))
@@ -399,7 +463,8 @@
                        "\n\n")))))))))
 
 (defun jira-detail--watchers (key callback)
-  "Show the watchers list of issue with KEY."
+  "Show the watchers list of issue with KEY.
+CALLBACK is called with the watchers data."
   (jira-api-call "GET"
                  (format "issue/%s/watchers" key)
                  :callback callback))
@@ -411,6 +476,7 @@
    :callback (lambda (data _) (jira-detail--issue key data))))
 
 (defun jira-detail--remove-comment-at-point ()
+  "Remove the comment at point."
   (let ((current-section (magit-current-section)))
     (if current-section
 	(if (string-equal (caar (magit-section-ident current-section)) "comment")
@@ -419,13 +485,12 @@
                 (jira-actions-delete-comment
 		 jira-detail--current-key
                  comment-id
-		 (lambda () (jira-detail-show-issue jira-detail--current-key))))
-	      )
+		 (lambda () (jira-detail-show-issue jira-detail--current-key)))))
 	  (message "No comment at point"))
       (message "No comment at point"))))
 
 (defun jira-detail--update-field-action (field-id value key)
-  ;; Update FIELD-ID with VALUE for the current issue.
+  "Update FIELD-ID with VALUE for the current issue KEY."
   (jira-api-call
    "PUT" (concat "issue/" key)
    :data `(("fields" . ((,field-id . ,value))))
@@ -439,23 +504,31 @@
    :callback (lambda (data _response) (jira-detail-show-issue (alist-get 'key data)))))
 
 (defun jira-detail--update-timetracking-action (field-id value key)
-  ;; Update timetracking FIELD-ID with VALUE for the current issue.
+  "Update timetracking FIELD-ID with VALUE for the current issue KEY."
   (jira-api-call
    "PUT" (concat "issue/" key)
    :data `(("fields" . (("timetracking" . ((,field-id . ,value))))))
    :callback (lambda (_data _response) (jira-detail-show-issue key))))
 
 (defun jira-detail--ensure-update-metadata ()
-  ;; Ensure jira-detail--current-update-metadata is populated and return it.
+  "Ensure jira-detail--current-update-metadata is populated and return it."
   (unless jira-detail--current-update-metadata
-    (setq jira-detail--current-update-metadata
-          (jira-api-get-create-metadata
-           (jira-detail--project-key)
-           (jira-table-extract-field jira-issues-fields :issue-type-id jira-detail--current)
-           :sync t)))
+    (let* ((issue-type-id (jira-table-extract-field
+			   jira-issues-fields
+			   :issue-type-id jira-detail--current)))
+      (if (null issue-type-id)
+          (progn
+	    (message "Warning: No issue type ID found for current issue")
+	    (setq jira-detail--current-update-metadata nil))
+        (setq jira-detail--current-update-metadata
+              (jira-api-get-create-metadata
+               (jira-detail--project-key)
+               issue-type-id
+               :sync t)))))
   jira-detail--current-update-metadata)
 
 (defun jira-detail--updatable-field-id (field-name)
+  "Get the updatable field ID for FIELD-NAME."
   (let ((id (cdr (assoc field-name jira-detail--updatable-fields))))
     (if (and (consp id) (eq (car id) :custom))
         (cdr (assoc (cdr id) jira-fields))
@@ -515,7 +588,7 @@
     (lambda () (interactive ) (jira-detail--remove-comment-at-point)))]
   ["Issue Actions"
    ("C" "Change issue status"
-    (lambda () (interactive) (jira-actions-change-issue-menu)))
+    (lambda () (interactive) (call-interactively #'jira-actions-change-issue-menu)))
    ("O" "Open issue in browser"
     (lambda () (interactive) (jira-actions-open-issue jira-detail--current-key)))
    ("P" "Show parent issue" (lambda () (interactive) (jira-detail--show-parent-issue)))
@@ -544,7 +617,7 @@
 		  (interactive) (jira-detail--remove-comment-at-point)))
     (define-key map (kbd "C")
 		(lambda () "Change issue status"
-		  (interactive) (jira-actions-change-issue-menu)))
+		  (interactive) (call-interactively #'jira-actions-change-issue-menu)))
     (define-key map (kbd "O")
 		(lambda () "Open issue in browser"
 		  (interactive)  (jira-actions-open-issue jira-detail--current-key)))
@@ -562,6 +635,7 @@
     (define-key map (kbd "g")
 		(lambda () "Refresh issue detail"
 		  (interactive) (jira-detail-show-issue jira-detail--current-key)))
+    (define-key map (kbd "h") 'jira-detail--actions-menu)
     (define-key map (kbd "P")
 		(lambda () "Show parent issue"
 		  (interactive) (jira-detail--show-parent-issue)))
@@ -571,7 +645,8 @@
   "Keymap for Jira Issue Detail buffers.")
 
 (defun jira-detail--show-parent-issue ()
-  "Show the detail view of the parent issue."
+  "Show the detail view of the parent issue.
+Shows the detail view of the parent issue for the current issue."
   (interactive)
   (let ((parent-key (jira-detail--issue-fmt jira-detail--current :parent-key)))
     (if (and parent-key (not (string= parent-key "")))
@@ -583,7 +658,7 @@
  choose one if there are multiple options."
   (let* ((project (jira-detail--project-key))
          (response (jira-api-get-project-issue-types project :sync t))
-         (types (alist-get 'issuetypes (aref (alist-get 'projects response) 0)))
+         (types (alist-get 'issueTypes response))
          (subtasks-types
           (cl-remove-if-not (lambda (item) (eq (cdr (assoc 'subtask item)) t))
 			    (append types nil))))
